local LFGFilter_VERSION = "0.3";
local LFGFilter_LOADED_TEXT = "LFGFilter Loaded.";
local LFGFilter_IsHooked = false
-- default global pattern list.
LFGFilter_PATTERN_LIST_DEFAULTS = { "[Ll][Ff][Gg]", "[Ll][Ff][0-9]*[Mm]", "LF" }
LFGFilter_PATTERN_LIST = LFGFilter_PATTERN_LIST_DEFAULTS
local LFGFilter_MyMessage = nil

-- called at load time.
function LFGFilter_OnLoad()
    LFGFilter_Initialize()
    SLASH_LFGFILTER1 = "/lfgfilter"
    SLASH_LFGFILTER2 = "/lfgf"
    SlashCmdList[ "LFGFILTER"] = LFGFilter_Slasher;
end

-- Called on any chat event.
function LFGFilter_ChatFrame_AddMessage( frame, msg, a1, a2, a3, a4, a5, a6, a7, a8, a9 )
    if not ( LFGFilter_IsHooked ) or ( msg == LFGFilter_MyMessage ) or ( string.find( msg, "LookingForGroup" ) == nil ) then
        return frame:LFGFilter_Orig_AddMessage( msg, a1, a2, a3, a4, a5, a6, a7, a8, a9 )
    else
        LFGFilter_Debug( "in addmessage" )
        if  ( LFGFilter_LetMessagePass( msg ) ~= true ) then
            return
--            return frame:LFGFilter_Orig_AddMessage( "FILTERED  !! "..msg, a1, a2, a3, a4, a5, a6, a7, a8, a9 )
        else
            return frame:LFGFilter_Orig_AddMessage( msg, a1, a2, a3, a4, a5, a6, a7, a8, a9 )
        end
    end
end

-- hook into the addmessage method.  Only call this once!
function LFGFilter_Initialize()
    if ( LFGFilter_IsHooked ~= true ) then
        LFGFilter_SendMessage( "Hooking" )
        for i=1,NUM_CHAT_WINDOWS do
            local cf = getglobal( "ChatFrame"..i);
            cf.LFGFilter_Orig_AddMessage = cf.AddMessage;
            cf.AddMessage = LFGFilter_ChatFrame_AddMessage;
        end
    end
    LFGFilter_Hook()
end


function LFGFilter_Hook()
    LFGFilter_IsHooked = true
    LFGFilter_SendMessage( "LFGFilter "..LFGFilter_VERSION.." On.  Use /lfgf or lfgfilter for info" )
end

-- unhook our event from the chat frame.
function LFGFilter_UnHook()
    LFGFilter_IsHooked = false
    LFGFilter_SendMessage( "LFGFilter Disabled" )
--    if ( LFGFilter_IsHooked == true ) then
--        for i=1,NUM_CHAT_WINDOWS do
--            local cf = getglobal( "ChatFrame"..i);
--            cf.AddMessage = cf.LFGFilter_Orig_AddMessage
--        end
--    end
--    LFGFilter_IsHooked = false
--    LFGFilter_SendMessage( "LFGFilter Unhooked." )
end

-- handle slash commands.
function LFGFilter_Slasher(cmd)
    if ( string.find( cmd, "^list" ) ~= nil ) then
        LFGFilter_ViewPatterns()
    elseif (string.find( cmd, "^add" ) ~= nil ) then
        LFGFilter_AddPattern( string.sub( cmd, 5 ))
    elseif ( string.find( cmd, "^remove" ) ~= nil ) then
        LFGFilter_RemovePattern( string.sub( cmd, 8 ))
    elseif ( string.find( cmd, "^off" ) ~= nil ) then
        LFGFilter_UnHook()
    elseif ( string.find( cmd, "^on" ) ~= nil ) then
        LFGFilter_Hook()
    elseif ( string.find( cmd, "^default" ) ~= nil ) then
        LFGFilter_RestoreDefaults()
    else
        LFGFilter_info()
    end
end

-- send info to the user
function LFGFilter_info()
    LFGFilter_SendMessage( "LFGFilter version "..LFGFilter_VERSION )
    LFGFilter_SendMessage( "   '/lfgf list' to list current patterns" )
    LFGFilter_SendMessage( "   '/lfgf add <pattern>' to add a new acceptance pattern" )
    LFGFilter_SendMessage( "   '/lfgf remove <pattern number>' to remove an acceptance pattern" )
    LFGFilter_SendMessage( "   '/lfgf default' to restore defaults patterns." )
    LFGFilter_SendMessage( "   '/lfgf off' turn the filter off" )
    LFGFilter_SendMessage( "   '/lfgf on' turn the filter on" )
end

-- test the message against the patterns, and if it matches, return true.
function LFGFilter_LetMessagePass( msg )
    -- if they want to filter on looking, they get everything because the channel name is in the text.
    local channelend = string.find( msg, "]" )
    local tmpmsg = msg
    if ( channelend ~= nil ) then
        tmpmsg = strsub( msg, channelend+1 )
    end
    LFGFilter_Debug( "testing msg "..tmpmsg )

    for i,pat in LFGFilter_PATTERN_LIST do
        if ( string.find( tmpmsg, pat ) ~= nil ) then
            LFGFilter_Debug( "matches "..pat )
            return true
        else
            LFGFilter_Debug( "does not match "..pat )
        end
    end
    return false
end

-- add a pattern to our global pattern list.
function LFGFilter_AddPattern( pattern )
    tinsert( LFGFilter_PATTERN_LIST, pattern )
    LFGFilter_SendMessage( pattern.." added" )
end

-- remove a pattern by index from our global pattern list.
function LFGFilter_RemovePattern( num )
    local realnum = tonumber( num )
    if ( realnum ~= nil ) and ( realnum > 0 ) and ( realnum <= getn(LFGFilter_PATTERN_LIST)) then
        LFGFilter_SendMessage( "removing pattern "..LFGFilter_PATTERN_LIST[realnum] )
        tremove( LFGFilter_PATTERN_LIST, num )
    else
        LFGFilter_SendMessage( "unable to remove pattern at index "..num )
    end
end

-- view all patterns in our pattern list.
function LFGFilter_ViewPatterns()
    LFGFilter_SendMessage( "LFGFilter Patterns:" )
    foreachi( LFGFilter_PATTERN_LIST, function(k,v) LFGFilter_SendMessage( k.." - "..v ) end )
end

-- restore default patterns.
function LFGFilter_RestoreDefaults()
    LFGFilter_SendMessage( "LFGFilter patterns restored to default." )
    LFGFilter_PATTERN_LIST = LFGFilter_PATTERN_LIST_DEFAULTS
end


-- send a message to the default chat frame.
function LFGFilter_SendMessage( msg )
    LFGFilter_MyMessage = msg
    DEFAULT_CHAT_FRAME:AddMessage( msg, 192/255, 192/255, 0/255, 1.0, DEFAULT_CHAT_FRAME:GetFadeDuration())
end

-- set the false to true to get debug messages.
function LFGFilter_Debug( msg )
    if ( false ) then
        LFGFilter_SendMessage( msg )
    end
end
