--[[

SQUIDMOD - the Griffon killer

Author/Artist: SAM

Additional thanks go to:
	Vjeux (SetTexture code)
	Ecksen (for helping me convert images)
	SinfulCreature (contributing artist)

Version history:

1.7.2
	The 1.7.1 fix didn't work - possibly because I forgot to change the Interface .toc number
	Changed ## SavedVariables into ## SavedVariablesPerCharacter
	Updated the Interface version number. (11000)
1.7.1
	Fix for 1.10 patch
1.7
	New EndCap added: "guardian" created by Sinful Creature
	Updated the Interface version number. (1800)
1.6
	New EndCap added: "whelp"
	Updated the Interface version number. (1700)
1.5.3
	Updated the Interface version number. (1600)
1.5.2
	Updated the Interface version number. (1500)
1.5.1
	Updated the Interface version number. (1300)
1.5
	Alliance Racial themes.
	Help text is now a bit more compact
1.4
	Horde Racial themes.
	A second murloc added.
	The old forsaken theme is now renamed as "skull"
1.3.1
	BUGFIX: Opening and closing the map does no longer hide or unhide the graphics.
1.3
	It is now possible to hide the end cap graphics alltogheter with command /squid hide.
	Guild Emblem Templates are now available. README file explains how to use them
1.2
	BUGFIX: Settings are now saved over sessions.
	Murloc graphic added.
	Forsaken racial theme added.
1.1
	Toggle functionality
	It is now possible to toggle between the default octopus and the 2 blizzard
	supplied default graphics with command /squid toggle
1.0
	Made into an AddOn by Vjeux
beta:
	original consept and art by SAM

]]

--------------------------
-- Global variables
--------------------------

SquidMod_DEFAULT_TOGGLE = 2;

-- patch 1.10 fix - 1.10 made RegisterForSave() illegal
--RegisterForSave("SquidModSettings");

--------------------------
-- OnLoad
--------------------------

function SquidMod_OnLoad()
	
	-- get stored data
	SquidMod_LoadVariables();
	
	-- register events
	this:RegisterEvent("VARIABLES_LOADED");

	-- register the slash command
	SLASH_SQUID1 = "/squid";
	SlashCmdList["SQUID"] = function(msg)
		SquidMod_SlashCommand(msg);
	end

	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("SquidMod v1.7 AddOn loaded");
	end
	UIErrorsFrame:AddMessage("SquidMod v1.7 AddOn loaded", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
	
end

--------------------------
-- OnEvent
--------------------------

function SquidMod_OnEvent()

	if(event == "VARIABLES_LOADED") then
		SquidMod_Update();
	end
	
end

--------------------------
-- OnUpdate
--------------------------

function SquidMod_OnUpdate()

	SquidMod_Update();
	
end

--------------------------
-- Load saved variables
--------------------------

function SquidMod_LoadVariables()

	if not SquidModSettings then
		
		-- default settings
		SquidModSettings = { };
		SquidModSettings.toggle = SquidMod_DEFAULT_TOGGLE;
		
	end

end

--------------------------
-- Handle Slash commands
--------------------------

function SquidMod_SlashCommand(msg)

	-- check the command
	if(msg) then
		
		local command = string.lower(msg);
		
		if(command == "toggle") then
		
			if (SquidModSettings.toggle == 18) then
				SquidModSettings.toggle = 2;
			else
				SquidModSettings.toggle = SquidModSettings.toggle + 1;
			end
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Toggled");
		
		elseif(command == "hide") then
			SquidModSettings.toggle = 0;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Hidden (use /squid unhide to show the end caps again)");
		elseif(command == "unhide") then
			SquidModSettings.toggle = 1;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Unhidden");
		elseif(command == "octopus") then
			SquidModSettings.toggle = 2;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Octopus");
		elseif(command == "griffon") then
			SquidModSettings.toggle = 3;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Blizz Default");
		elseif(command == "lion") then
			SquidModSettings.toggle = 4;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Blizz Lion");
		elseif(command == "murloc1") then
			SquidModSettings.toggle = 5;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Murloc 1");
		elseif(command == "murloc2") then
			SquidModSettings.toggle = 6;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Murloc 2");
		elseif(command == "skull") then
			SquidModSettings.toggle = 7;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Skull");
		elseif(command == "whelp") then
			SquidModSettings.toggle = 8;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Whelp");
		elseif(command == "emblem") then
			SquidModSettings.toggle = 9;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Guild Emblem");
		elseif(command == "human") then
			SquidModSettings.toggle = 10;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Human theme");
		elseif(command == "dwarf") then
			SquidModSettings.toggle = 11;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Dwarf theme");
		elseif(command == "gnome") then
			SquidModSettings.toggle = 12;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Gnome theme");
		elseif(command == "nightelf") then
			SquidModSettings.toggle = 13;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Nightelf theme");
		elseif(command == "orc") then
			SquidModSettings.toggle = 14;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Orc theme");
		elseif(command == "troll") then
			SquidModSettings.toggle = 15;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Troll theme");
		elseif(command == "tauren") then
			SquidModSettings.toggle = 16;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Tauren theme");
		elseif(command == "forsaken") then
			SquidModSettings.toggle = 17;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Forsaken theme");
		elseif(command == "guardian") then
			SquidModSettings.toggle = 18;
			SquidMod_Update();
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Guardian (created by SinfulCreature)");
		
		else
			DEFAULT_CHAT_FRAME:AddMessage("-------------------------------------------------");
			DEFAULT_CHAT_FRAME:AddMessage("SquidMod Command List");
			DEFAULT_CHAT_FRAME:AddMessage(" ");
			DEFAULT_CHAT_FRAME:AddMessage("/squid toggle - Toggles the graphics");
			DEFAULT_CHAT_FRAME:AddMessage("/squid hide - Hides the end cap graphics");
			DEFAULT_CHAT_FRAME:AddMessage("/squid unhide - Shows the end cap graphics");
			DEFAULT_CHAT_FRAME:AddMessage(" ");
			DEFAULT_CHAT_FRAME:AddMessage("Misc graphics: type /squid followed by");
			DEFAULT_CHAT_FRAME:AddMessage("octopus, murloc1, murloc2, skull, whelp, guardian");
			DEFAULT_CHAT_FRAME:AddMessage(" ");
			DEFAULT_CHAT_FRAME:AddMessage("Original Blizzard graphics: type /squid followed by");
			DEFAULT_CHAT_FRAME:AddMessage("griffon or lion");
			DEFAULT_CHAT_FRAME:AddMessage(" ");
			DEFAULT_CHAT_FRAME:AddMessage("Guild Emblem Template: type /squid emblem");
			DEFAULT_CHAT_FRAME:AddMessage("see the README supplied with this mod for instructions on how to use the Photoshop templates");
			DEFAULT_CHAT_FRAME:AddMessage(" ");
			DEFAULT_CHAT_FRAME:AddMessage("Alliance racial themes: type /squid followed by");
			DEFAULT_CHAT_FRAME:AddMessage("human, dwarf, gnome or nightelf");
			DEFAULT_CHAT_FRAME:AddMessage(" ");
			DEFAULT_CHAT_FRAME:AddMessage("Horde racial themes: type /squid followed by");
			DEFAULT_CHAT_FRAME:AddMessage("orc, troll, tauren or forsaken");
			DEFAULT_CHAT_FRAME:AddMessage("-------------------------------------------------");
			
		end
	
	end

end

--------------------------
-- Update the EndCaps
--------------------------

function SquidMod_Update()

	if(SquidModSettings) then
			
		if(SquidModSettings.toggle == 0) then
			MainMenuBarLeftEndCap:Hide();
			MainMenuBarRightEndCap:Hide();
		elseif(SquidModSettings.toggle == 1) then
			MainMenuBarLeftEndCap:Show();
			MainMenuBarRightEndCap:Show();
			SquidModSettings.toggle = 2;
			SquidMod_Update();
		elseif(SquidModSettings.toggle == 2) then
			-- Thanx to Vjeux for SetTexture code
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\SquidModDefault.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\SquidModDefault.tga");
		elseif(SquidModSettings.toggle == 3) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\BlizzDefault1.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\BlizzDefault1.tga");
		elseif(SquidModSettings.toggle == 4) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\BlizzDefault2.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\BlizzDefault2.tga");
		elseif(SquidModSettings.toggle == 5) then	
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\murloc.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\murloc.tga");
		elseif(SquidModSettings.toggle == 6) then	
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\murloc2.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\murloc2.tga");
		elseif(SquidModSettings.toggle == 7) then	
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\skull.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\skull.tga");
		elseif(SquidModSettings.toggle == 8) then	
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\whelp.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\whelp.tga");
		elseif(SquidModSettings.toggle == 9) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\emblemLeft.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\emblemRight.tga");
		elseif(SquidModSettings.toggle == 10) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\human.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\human.tga");
		elseif(SquidModSettings.toggle == 11) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\dwarf.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\dwarf.tga");
		elseif(SquidModSettings.toggle == 12) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\gnome.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\gnome.tga");
		elseif(SquidModSettings.toggle == 13) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\nightelf.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\nightelf.tga");
		elseif(SquidModSettings.toggle == 14) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\orc.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\orc.tga");
		elseif(SquidModSettings.toggle == 15) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\troll.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\troll.tga");
		elseif(SquidModSettings.toggle == 16) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\tauren.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\tauren.tga");
		elseif(SquidModSettings.toggle == 17) then
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\ghoul.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\ghoul.tga");
		else
			MainMenuBarLeftEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\SinfulCreature_guardian.tga");
			MainMenuBarRightEndCap:SetTexture("Interface\\AddOns\\SquidMod\\skin\\SinfulCreature_guardian.tga");
		end
	
	end

end